@@ -16,7 +16,9 @@
 Logilab common libraries
 """
 
-__revision__ = "$Id: __init__.py,v 1.26 2005/08/08 10:00:11 adim Exp $"
+from __future__ import nested_scopes
+
+__revision__ = "$Id: __init__.py,v 1.30 2006-03-24 10:37:48 syt Exp $"
 
 # FIXME: move all those functions in a separated module
 
@@ -134,6 +136,80 @@ def _get_cycles(graph_dict, vertice=None, path=None, result=None):
     path.pop()
 
 
+def cached(callableobj, keyarg=None):
+    """simple decorator to cache result of method call"""
+    #print callableobj, keyarg, callableobj.func_code.co_argcount
+    if callableobj.func_code.co_argcount == 1 or keyarg == 0:
+        
+        def cache_wrapper1(self, *args):
+            cache = '_%s_cache_' % callableobj.__name__
+            #print 'cache1?', cache
+            try:
+                return getattr(self, cache)
+            except AttributeError:
+                #print 'miss'
+                value = callableobj(self, *args)
+                setattr(self, cache, value)
+                return value
+        return cache_wrapper1
+    
+    elif keyarg:
+        
+        def cache_wrapper2(self, *args, **kwargs):
+            cache = '_%s_cache_' % callableobj.__name__
+            key = args[keyarg-1]
+            #print 'cache2?', cache, self, key
+            try:
+                _cache = getattr(self, cache)
+            except AttributeError:
+                #print 'init'
+                _cache = {}
+                setattr(self, cache, _cache)
+            try:
+                return _cache[key]
+            except KeyError:
+                #print 'miss', self, cache, key
+                _cache[key] = callableobj(self, *args, **kwargs)
+            return _cache[key]
+        return cache_wrapper2
+    def cache_wrapper3(self, *args):
+        cache = '_%s_cache_' % callableobj.__name__
+        #print 'cache3?', cache, self, args
+        try:
+            _cache = getattr(self, cache)
+        except AttributeError:
+            #print 'init'
+            _cache = {}
+            setattr(self, cache, _cache)
+        try:
+            return _cache[args]
+        except KeyError:
+            #print 'miss'
+            _cache[args] = callableobj(self, *args)
+        return _cache[args]
+    return cache_wrapper3
+
+import sys
+
+class ProgressBar(object):
+    """a simple text progression bar"""
+    
+    def __init__(self, nbops, size=20., stream=sys.stdout):
+        self._dotevery = max(nbops / size, 1)
+        self._fstr = '\r[%-20s]'
+        self._dotcount, self._dots = 1, []
+        self._stream = stream
+
+    def update(self):
+        """update the progression bar"""
+        self._dotcount += 1
+        if self._dotcount >= self._dotevery:
+            self._dotcount = 1
+            self._dots.append('.')
+            self._stream.write(self._fstr % ''.join(self._dots))
+            self._stream.flush()
+
+
 import tempfile
 import os
 import time
@@ -178,12 +254,15 @@ def release_lock(lock_file):
 from warnings import warn
 
 class deprecated(type):
+    """metaclass to print a warning on instantiation of a deprecated class"""
+    
     def __call__(cls, *args, **kwargs):
         msg = getattr(cls, "__deprecation_warning__",
                       "%s is deprecated" % cls.__name__)
         warn(msg, DeprecationWarning, stacklevel=2)
         return type.__call__(cls, *args, **kwargs)
 
+
 def class_renamed(old_name, new_class, message=None):
     """automatically creates a class which fires a DeprecationWarning
     when instantiated.
@@ -200,10 +279,11 @@ def class_renamed(old_name, new_class, message=None):
     try:
         # new-style class
         return deprecated(old_name, (new_class,), clsdict)
-    except TypeError:
-        # FIXME: There might be a better way to handle old/new-style class
+    except (NameError, TypeError):
         # old-style class
         class DeprecatedClass(new_class):
+            """FIXME: There might be a better way to handle old/new-style class
+            """
             def __init__(self, *args, **kwargs):
                 warn(message, DeprecationWarning, stacklevel=2)
                 new_class.__init__(self, *args, **kwargs)
