@@ -1,4 +1,4 @@
-# Copyright (c) 2003-2005 LOGILAB S.A. (Paris, FRANCE).
+# Copyright (c) 2003-2006 LOGILAB S.A. (Paris, FRANCE).
 # http://www.logilab.fr/ -- mailto:contact@logilab.fr
 
 # This program is free software; you can redistribute it and/or modify it under
@@ -13,18 +13,16 @@
 # You should have received a copy of the GNU General Public License along with
 # this program; if not, write to the Free Software Foundation, Inc.,
 # 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
-"""
-logilab.common packaging information
-"""
+"""logilab.common packaging information"""
 
-__revision__ = "$Id: __pkginfo__.py,v 1.48 2005/09/06 09:45:28 alf Exp $"
+__revision__ = "$Id: __pkginfo__.py,v 1.58 2006-04-25 12:08:52 syt Exp $"
 
 modname = 'common'
-numversion = (0, 12, 0)
+numversion = (0, 16, 1)
 version = '.'.join([str(num) for num in numversion])
 
 license = 'GPL'
-copyright = '''Copyright (c) 2003-2005 LOGILAB S.A. (Paris, FRANCE).
+copyright = '''Copyright (c) 2003-2006 LOGILAB S.A. (Paris, FRANCE).
 http://www.logilab.fr/ -- mailto:contact@logilab.fr'''
 
 author = "Logilab"
@@ -32,7 +30,8 @@ author_email = "devel@logilab.fr"
 
 short_desc = "useful miscellaneous modules used by Logilab projects"
 
-long_desc = """logilab-common is a collection of low-level Python packages and modules,
+long_desc = """logilab-common is a collection of low-level Python packages and \
+modules,
  designed to ease:
   * handling command line options and configuration files
   * writing interactive command line tools
@@ -51,11 +50,10 @@ ftp = "ftp://ftp.logilab.org/pub/%s" % modname
 mailinglist = "mailto://python-projects@lists.logilab.org"
 
 subpackage_of = 'logilab'
+subpackage_master = True
 
 from os.path import join
 include_dirs = [join('test', 'data')]
-
-debian_name = 'logilab-common'
+pyversions = ['2.2', '2.3', '2.4']
 debian_maintainer = 'Alexandre Fayolle'
 debian_maintainer_email = 'afayolle@debian.org'
-debian_handler = 'python-library'
