@@ -18,7 +18,7 @@
  their names to values provided in a dictionnary
 """
 
-__revision__ = '$Id: bind.py,v 1.7 2005/02/15 17:06:08 adim Exp $'
+__revision__ = '$Id: bind.py,v 1.8 2005-11-22 13:12:59 syt Exp $'
 
 # TODO: unit tests
 # * this module provide a function bind(func,vars) which replaces every
@@ -58,7 +58,7 @@ def bind_code(co, globals):
         if op == LOAD_GLOBAL:
             name = co.co_names[oparg]
             if globals.has_key(name):
-                k = assigned.get(name,None)
+                k = assigned.get(name, None)
                 if k == None:
                     k = len(consts)
                     assigned[name] = len(consts)
@@ -149,7 +149,7 @@ def analyze_code(co, globals, consts_dict, consts_list):
         if op == LOAD_GLOBAL:
             name = co.co_names[oparg]
             if globals.has_key(name):
-                k = consts_dict.get(name,None)
+                k = consts_dict.get(name, None)
                 if k == None:
                     k = len(consts_list)
                     consts_dict[name] = k
@@ -244,12 +244,12 @@ def run_bench(n):
     t = time()
     g = globals()
     for i in range(n):
-        test = bind(bind_code,g)
+        test = bind(bind_code, g)
     t1 = time()-t
     bind2 = bind(bind, {'bind_code':bind_code_opt})
     t = time()
     for i in range(n):
-        test=bind2(bind_code,g)
+        test=bind2(bind_code, g)
     t2 = time()-t
     print "1 regular version", t1
     print "2 optimized version", t2
@@ -258,13 +258,13 @@ def run_bench(n):
 
 def test_pystone():
     from test import pystone
-    for i in range(5):
+    for _ in range(5):
         pystone.main()
-    optimize_module(pystone,('TRUE','FALSE','Proc0','Proc1','Proc2','Proc3',
-                             'Proc4','Proc5','Proc6','Proc7','Proc8','Func1',
-                             'Func2','Func3'))
+    optimize_module(pystone, ('TRUE','FALSE','Proc0','Proc1','Proc2','Proc3',
+                              'Proc4','Proc5','Proc6','Proc7','Proc8','Func1',
+                             ' Func2','Func3'))
     optimize_module(pystone, builtins.keys())
-    for i in range(5):
+    for _ in range(5):
         pystone.main()
 
 
