@@ -1,4 +1,4 @@
-# Copyright (c) 2003-2005 LOGILAB S.A. (Paris, FRANCE).
+# Copyright (c) 2003-2006 LOGILAB S.A. (Paris, FRANCE).
 # http://www.logilab.fr/ -- mailto:contact@logilab.fr
 #
 # This program is free software; you can redistribute it and/or modify it under
@@ -24,7 +24,7 @@ in the `optik_ext` module)
 
 
 Quick start: simplest usage
-```````````````````````````````
+```````````````````````````
 
 import sys
 from logilab.common.configuration import Configuration
@@ -57,37 +57,40 @@ print config['value']
 print config['multiple']
 print config['number']
 
-sys.argv = ['mon prog', '--value', 'bacon', '--multiple', '4,5,6', 'nonoptionargument']
+sys.argv = ['mon prog', '--value', 'bacon', '--multiple', '4,5,6',
+            'nonoptionargument']
 print config.load_command_line_configuration()
 print config['value']
 
 config.generate_config()
 
 
-:version:   $Revision: 1.37 $  
+:version:   $Revision: 1.40 $  
 :author:    Logilab
-:copyright: 2003-2005 LOGILAB S.A. (Paris, FRANCE)
+:copyright: 2003-2006 LOGILAB S.A. (Paris, FRANCE)
 :contact:   http://www.logilab.fr/ -- mailto:python-projects@logilab.org
 """
 
-__revision__ = "$Id: configuration.py,v 1.37 2005/04/11 10:50:42 syt Exp $"
+from __future__ import generators 
+
+__revision__ = "$Id: configuration.py,v 1.40 2005-11-22 13:13:00 syt Exp $"
 __docformat__ = "restructuredtext en"
 __all__ = ('OptionsManagerMixIn', 'OptionsProviderMixIn',
            'ConfigurationMixIn', 'Configuration',
            'OptionsManager2ConfigurationAdapter')
 
-
+import os
 import sys
 import re
-from os import linesep
 from os.path import exists
 from copy import copy
 from ConfigParser import ConfigParser, NoOptionError, NoSectionError
 
-from logilab.common.optik_ext import OptionParser, OptionGroup, Values, \
-     OptionValueError, OptionError, check_yn, check_csv, check_file, \
-     check_color, check_named, generate_manpage
 from logilab.common.textutils import normalize_text, unquote
+from logilab.common.optik_ext import OptionParser, OptionGroup, Values, \
+     OptionValueError, OptionError, HelpFormatter, generate_manpage, \
+     check_yn, check_csv, check_file, check_color, check_named, \
+     NO_DEFAULT, OPTPARSE_FORMAT_DEFAULT
 
 class UnsupportedAction(Exception):
     """raised by set_option when it doesn't know what to do for an action"""
@@ -150,6 +153,28 @@ VALIDATORS = {'string' : unquote,
               'multiple_choice': multiple_choice_validator,
               }
 
+def expand_default(self, option):
+    """monkey patch OptionParser.expand_default since we have a particular
+    way to handle defaults to avoid overriding values in the configuration
+    file
+    """
+    if self.parser is None or not self.default_tag:
+        return option.help
+    optname = option._long_opts[0][2:]
+    try:
+        provider = self.parser.options_manager._all_options[optname]
+    except KeyError:
+        value = None
+    else:
+        optdict = provider.get_option_def(optname)
+        optname = provider.option_name(optname, optdict)
+        value = getattr(provider.config, optname, optdict)
+        value = format_option_value(optdict, value)
+    if value is NO_DEFAULT or not value:
+        value = self.NO_DEFAULT_VALUE
+    return option.help.replace(self.default_tag, str(value))
+
+
 def convert(value, opt_dict, name=''):
     """return a validated value for an option according to its type
     
@@ -175,31 +200,56 @@ def convert(value, opt_dict, name=''):
 
 def comment(string):
     """return string as a comment"""
-    return '# ' + string.replace('\n', '\n# ')
-                
-def format_section(stream, section, options, doc=None):
+    lines = [line.strip() for line in string.splitlines()]
+    return '# ' + ('%s# ' % os.linesep).join(lines)
+
+def format_option_value(optdict, value):
+    """return the user input's value from a 'compiled' value"""
+    if type(value) in (type(()), type([])):
+        value = ','.join(value)
+    elif hasattr(value, 'match'): # optdict.get('type') == 'regexp'
+        # compiled regexp
+        value = value.pattern
+    elif optdict.get('type') == 'yn':
+        value = value and 'yes' or 'no'
+    elif isinstance(value, (str, unicode)) and value.isspace():
+        value = "'%s'" % value
+    return value
+
+def ini_format_section(stream, section, options, doc=None):
     """format an options section using the INI format"""
     if doc:
         print >> stream, comment(doc)
     print >> stream, '[%s]' % section.upper()
     section = {}
-    for opt_name, value, opt_dict in options:
-        if type(value) in (type(()), type([])):
-            value = ','.join(value)
-        elif hasattr(value, 'match'):
-            # compiled regexp
-            value = value.pattern
-        elif opt_dict.get('type') == 'yn':
-            value = value and 'yes' or 'no'
-        elif isinstance(value, (str, unicode)) and value.isspace():
-            value = "'%s'" % value
-        #else:
-        #    value = repr(value)
-        help_msg = opt_dict.get('help')
-        if help_msg:
-            print >> stream, normalize_text(help_msg, indent='# ')
-        print >> stream, '%s=%s\n' % (opt_name, value)
-    print >> stream, '\n'
+    for optname, optdict, value in options:
+        if value is None:
+            continue
+        value = format_option_value(optdict, value)
+        help = optdict.get('help')
+        if help:
+            print >> stream
+            print >> stream, normalize_text(help, line_len=79, indent='# ')
+        else:
+            print >> stream
+        print >> stream, '%s=%s' % (optname, str(value).strip())
+        
+format_section = ini_format_section
+
+def rest_format_section(stream, section, options, doc=None):
+    """format an options section using the INI format"""
+    if section:
+        print >> stream, '%s\n%s' % (section, "'"*len(section))
+    if doc:
+        print >> stream, normalize_text(doc, line_len=79, indent='')
+        print >> stream
+    for optname, optdict, value in options:
+        help = optdict.get('help')
+        print >> stream, ':%s:' % optname
+        if help:
+            print >> stream, normalize_text(help, line_len=79, indent='  ')
+        if value:
+            print >> stream, '  Default: %s' % format_option_value(optdict, value)
 
 
 class OptionsManagerMixIn:
@@ -213,6 +263,7 @@ class OptionsManagerMixIn:
         self._config_parser = ConfigParser()
         # command line parser
         self._optik_parser = OptionParser(usage=usage, version=version)
+        self._optik_parser.options_manager = self
         # list of registered options providers
         self.options_providers = []
         # dictionary assocating option name to checker
@@ -256,10 +307,10 @@ class OptionsManagerMixIn:
         # add section to the config file
         self._config_parser.add_section(group_name)
         # add option group to the command line parser
-        group = OptionGroup(self._optik_parser,
-                            title=group_name.capitalize(),
-                            description=doc)
-        self._optik_parser.add_option_group(group)
+        if options:
+            group = OptionGroup(self._optik_parser,
+                                title=group_name.capitalize())
+            self._optik_parser.add_option_group(group)
         # add provider's specific options
         for opt_name, opt_dict in options:
             args, opt_dict = self.optik_option(provider, opt_name, opt_dict)
@@ -279,6 +330,9 @@ class OptionsManagerMixIn:
         for specific in ('default', 'group'):
             if opt_dict.has_key(specific):
                 del opt_dict[specific]
+                if (OPTPARSE_FORMAT_DEFAULT
+                    and specific == 'default' and opt_dict.has_key('help')):
+                    opt_dict['help'] += ' [current: %default]'
         args = ['--' + opt_name]
         if opt_dict.has_key('short'):
             self._short_options[opt_dict['short']] = opt_name
@@ -288,10 +342,11 @@ class OptionsManagerMixIn:
             
     def cb_set_provider_option(self, option, opt_name, value, parser):
         """optik callback for option setting"""
-        # remove --
         if opt_name.startswith('--'):
+            # remove -- on long option
             opt_name = opt_name[2:]
-        else: # short version
+        else:
+            # short option, get its long equivalent
             opt_name = self._short_options[opt_name[1:]]
         # trick since we can't set action='store_true' on options
         if value is None:
@@ -307,29 +362,22 @@ class OptionsManagerMixIn:
         into the given stream or stdout
         """
         stream = stream or sys.stdout
+        printed = False
         for provider in self.options_providers:
             default_options = []
             sections = {}
-            for opt_name, opt_dict in provider.options:
-                if opt_dict.get('type') is None:
-                    continue                
-                attr = provider.option_name(opt_name)
-                try:
-                    value = getattr(provider.config, attr)
-                except AttributeError:
-                    continue
-                if value is None:
-                    continue
-                if opt_dict.get('group'):
-                    sections.setdefault(opt_dict['group'], []).append(
-                        (opt_name, value, opt_dict))
+            for section, options in provider.options_by_section():
+                options = [(n, d, v) for (n, d, v) in options
+                           if d.get('type') is not None and v is not None]
+                if section is None:
+                    section = provider.name
+                    doc = provider.__doc__
                 else:
-                    default_options.append((opt_name, value, opt_dict))
-            if default_options:
-                format_section(stream, provider.name, default_options,
-                               provider.__doc__)
-            for section, options in sections.items():
-                format_section(stream, section, options)
+                    doc = None
+                if printed:
+                    print >> stream, '\n'
+                format_section(stream, section, options, doc)
+                printed = True
 
     def generate_manpage(self, pkginfo, section=1, stream=None):
         """write a man page for the current configuration into the given
@@ -343,15 +391,27 @@ class OptionsManagerMixIn:
     def load_file_configuration(self, config_file=None):
         """load the configuration from file
         """
+        self.read_config_file(config_file)
+        self.load_config_file()
+        
+    def read_config_file(self, config_file=None):
+        """read the configuration file but do not load it (ie dispatching
+        values to each options provider)
+        """
         if config_file is None:
             config_file = self.config_file
-        parser = self._config_parser
         if config_file and exists(config_file):
-            parser.read([config_file])
+            self._config_parser.read([config_file])
         elif not self.quiet:
             msg = 'No config file found, using default configuration'
             print >> sys.stderr, msg
             return
+        
+    def load_config_file(self):
+        """dispatch values previously read from a configuration file to each
+        options provider)
+        """
+        parser = self._config_parser        
         for provider in self.options_providers:
             default_section = provider.name
             for opt_name, opt_dict in provider.options:
@@ -377,19 +437,31 @@ class OptionsManagerMixIn:
 
         return additional arguments
         """
-        if args is None:
-            args = sys.argv[1:]
-        else:
-            args = list(args)
-        (options, args) = self._optik_parser.parse_args(args=args)
-        for provider in self._nocallback_options.keys():
-            config = provider.config
-            for attr in config.__dict__.keys():
-                value = getattr(options, attr, None)
-                if value is None:
-                    continue
-                setattr(config, attr, value)
-        return args
+        # monkey patch optparse to deal with our default values
+        try:
+            expand_default_backup = HelpFormatter.expand_default
+            HelpFormatter.expand_default = expand_default
+        except AttributeError:
+            # python < 2.4: nothing to be done
+            pass
+        try:
+            if args is None:
+                args = sys.argv[1:]
+            else:
+                args = list(args)
+            (options, args) = self._optik_parser.parse_args(args=args)
+            for provider in self._nocallback_options.keys():
+                config = provider.config
+                for attr in config.__dict__.keys():
+                    value = getattr(options, attr, None)
+                    if value is None:
+                        continue
+                    setattr(config, attr, value)
+            return args
+        finally:
+            if hasattr(HelpFormatter, 'expand_default'):
+                # unpatch optparse to avoid side effects
+                HelpFormatter.expand_default = expand_default_backup
 
 
     # help methods ############################################################
@@ -411,7 +483,7 @@ class OptionsProviderMixIn:
     """Mixin to provide options to an OptionsManager
     """
     
-    # those attributes should be overriden
+    # those attributes should be overridden
     priority = -1
     name = 'default'
     options = ()
@@ -435,6 +507,10 @@ class OptionsProviderMixIn:
         if opt_dict is None:
             opt_dict = self.get_option_def(opt_name)
         return opt_dict.get('dest', opt_name.replace('-', '_'))
+    
+    def option_value(self, opt_name):
+        """get the current value for the given option"""
+        return getattr(self.config, self.option_name(opt_name), None)
         
     def set_option(self, opt_name, value, action=None, opt_dict=None):
         """method called to set an option (registered in the options list)
@@ -446,13 +522,13 @@ class OptionsProviderMixIn:
         if action is None:
             action = opt_dict.get('action', 'store')
         if action == 'store':
-            setattr(self.config, self.option_name(opt_name), value)
+            setattr(self.config, self.option_name(opt_name, opt_dict), value)
         elif action in ('store_true', 'count'):
-            setattr(self.config, self.option_name(opt_name), 0)
+            setattr(self.config, self.option_name(opt_name, opt_dict), 0)
         elif action == 'store_false':
-            setattr(self.config, self.option_name(opt_name), 1)
+            setattr(self.config, self.option_name(opt_name, opt_dict), 1)
         elif action == 'append':
-            opt_name = self.option_name(opt_name)
+            opt_name = self.option_name(opt_name, opt_dict)
             _list = getattr(self.config, opt_name, None)
             if _list is None:
                 if type(value) in (type(()), type([])):
@@ -473,8 +549,21 @@ class OptionsProviderMixIn:
         for opt in self.options:
             if opt[0] == opt_name:
                 return opt[1]
-        else:
-            raise OptionError('no such option in section %r' % self.name, opt_name)
+        raise OptionError('no such option in section %r' % self.name, opt_name)
+
+    def options_by_section(self):
+        """return an iterator on options grouped by section
+        
+        (section, [list of (optname, optdict, optvalue)])
+        """
+        sections = {}
+        for optname, optdict in self.options:
+            sections.setdefault(optdict.get('group'), []).append(
+                (optname, optdict, self.option_value(optname)))
+        if None in sections:
+            yield None, sections.pop(None)
+        for section, options in sections.items():
+            yield section, options
        
 
 class ConfigurationMixIn(OptionsManagerMixIn, OptionsProviderMixIn):
@@ -497,7 +586,8 @@ class Configuration(ConfigurationMixIn):
     configuration values are accessible through a dict like interface
     """
 
-    def __init__(self, config_file=None, options=None, name=None, usage=None, doc=None):
+    def __init__(self, config_file=None, options=None, name=None,
+                 usage=None, doc=None):
         if options is not None:
             self.options = options
         if name is not None:
