@@ -1,4 +1,4 @@
-# Copyright (c) 2002-2005 LOGILAB S.A. (Paris, FRANCE).
+# Copyright (c) 2002-2006 LOGILAB S.A. (Paris, FRANCE).
 # http://www.logilab.fr/ -- mailto:contact@logilab.fr
 #
 # This program is free software; you can redistribute it and/or modify it under
@@ -13,16 +13,15 @@
 # You should have received a copy of the GNU General Public License along with
 # this program; if not, write to the Free Software Foundation, Inc.,
 # 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
-"""A generic function to get a database connection.
+"""Helpers to get a DBAPI2 compliant database connection.
 """
 
-__revision__ = "$Id: db.py,v 1.27 2005/08/08 10:42:24 adim Exp $"
+__revision__ = "$Id: db.py,v 1.35 2006-04-25 12:02:09 syt Exp $"
 
 import sys
+import re
 
-from logilab.common import deprecated_function
-
-__all__ = ['get_dbapi_compliant_module', 'get_connexion',
+__all__ = ['get_dbapi_compliant_module', 
            'get_connection', 'set_prefered_driver',
            'PyConnection', 'PyCursor',
            'UnknownDriver', 'NoAdapterFound',
@@ -43,12 +42,6 @@ class NoAdapterFound(Exception):
         self._protocol = protocol
 
 
-PREFERED_DRIVERS = {
-    "postgres" : [ 'psycopg', 'pgdb', 'pyPgSQL.PgSQL', ],
-    "mysql" : [ 'MySQLdb', ], # 'pyMySQL.MySQL, ],
-    "sqlite" : [ 'sqlite', ],
-    }
-
 def _import_driver_module(driver, drivers, imported_elements=None, quiet=True):
     """Imports the first module found in 'drivers' for 'driver'
 
@@ -71,28 +64,13 @@ def _import_driver_module(driver, drivers, imported_elements=None, quiet=True):
             continue
     else:
         raise ImportError('Unable to import a %s module' % driver)
+    if not imported_elements:
+        for part in modname.split('.')[1:]:
+            module = getattr(module, part)
     return module, modname
 
-def set_prefered_driver(database, module, _drivers=PREFERED_DRIVERS):
-    """sets the prefered driver module for database
-    database is the name of the db engine (postgresql, mysql...)
-    module is the name of the module providing the connect function
-    syntax is (params_func, post_process_func_or_None)
-    _drivers is a optionnal dictionnary of drivers
-    """
-    try:
-        modules = _drivers[database]
-    except KeyError:
-        raise UnknownDriver('Unknown database %s' % database)
-    # Remove module from modules list, and re-insert it in first position
-    try:
-        modules.remove(module)
-    except ValueError:
-        raise UnknownDriver('Unknown module %s for %s' % (module, database))
-    modules.insert(0, module)
 
-
-## Connection and cursor wrappers ####################################
+## Connection and cursor wrappers #############################################
         
 class PyConnection:
     """A simple connection wrapper in python (useful for profiling)"""
@@ -154,7 +132,7 @@ class PyCursor:
         return getattr(self._cursor, attrname)
     
 
-## Adapters list #####################################################
+## Adapters list ##############################################################
     
 class DBAPIAdapter:
     """Base class for all DBAPI adpaters"""
@@ -167,9 +145,9 @@ class DBAPIAdapter:
         self._native_module = native_module
         self._pywrap = pywrap
 
-    def connect(self, host = '', database = '', user = '', password = ''):
+    def connect(self, host='', database='', user='', password='', port=''):
         """Wraps the native module connect method"""
-        kwargs = {'host' : host, 'database' : database,
+        kwargs = {'host' : host, 'port' : port, 'database' : database,
                   'user' : user, 'password' : password}
         cnx = self._native_module.connect(**kwargs)
         return self._wrap_if_needed(cnx)
@@ -187,6 +165,8 @@ class DBAPIAdapter:
         return getattr(self._native_module, attrname)
 
 
+# Postgresql #########################################################
+
 class _PgdbAdapter(DBAPIAdapter):
     """Simple PGDB Adapter to DBAPI (pgdb modules lacks Binary() and NUMBER)
     """
@@ -196,16 +176,27 @@ class _PgdbAdapter(DBAPIAdapter):
                                              'int8', 'float4', 'float8',
                                              'numeric', 'bool', 'money')
         
+    def connect(self, host='', database='', user='', password='', port=''):
+        """Wraps the native module connect method"""
+        if port:
+            warn("pgdb doesn't support 'port' parameter in connect()", UserWarning)
+        kwargs = {'host' : host, 'database' : database,
+                  'user' : user, 'password' : password}
+        cnx = self._native_module.connect(**kwargs)
+        return self._wrap_if_needed(cnx)
+
 
 class _PsycopgAdapter(DBAPIAdapter):
     """Simple Psycopg Adapter to DBAPI (cnx_string differs from classical ones)
     """
-    def connect(self, host = '', database = '', user = '', password = ''):
+    def connect(self, host='', database='', user='', password='', port=''):
         """Handles psycopg connexion format"""
         if host:
             cnx_string = 'host=%s  dbname=%s  user=%s' % (host, database, user)
         else:
             cnx_string = 'dbname=%s  user=%s' % (database, user)
+        if port:
+            cnx_string += ' port=%s' % port
         if password:
             cnx_string = '%s password=%s' % (cnx_string, password)
         cnx = self._native_module.connect(cnx_string)
@@ -213,13 +204,18 @@ class _PsycopgAdapter(DBAPIAdapter):
         return self._wrap_if_needed(cnx)
     
 
+class _Psycopg2Adapter(_PsycopgAdapter):
+    """Simple Psycopg2 Adapter to DBAPI (cnx_string differs from classical ones)
+    """
+    BOOLEAN = 16 # XXX
+    
 
 class _PgsqlAdapter(DBAPIAdapter):
     """Simple pyPgSQL Adapter to DBAPI
     """
-    def connect(self, host = '', database = '', user = '', password = ''):
+    def connect(self, host='', database='', user='', password='', port=''):
         """Handles psycopg connexion format"""
-        kwargs = {'host' : host, 'database' : database,
+        kwargs = {'host' : host, 'port': port, 'database' : database,
                   'user' : user, 'password' : password or None}
         cnx = self._native_module.connect(**kwargs)
         return self._wrap_if_needed(cnx)
@@ -231,76 +227,126 @@ class _PgsqlAdapter(DBAPIAdapter):
     
     def __getattr__(self, attrname):
         # __import__('pyPgSQL.PgSQL', ...) imports the toplevel package
-        return getattr(self._native_module.PgSQL, attrname)
+        return getattr(self._native_module, attrname)
 
 
+# Sqlite #############################################################
+
+class _PySqlite2Adapter(DBAPIAdapter):
+    """Simple pysqlite2 Adapter to DBAPI
+    """
+    def __init__(self, native_module, pywrap=False):
+        DBAPIAdapter.__init__(self, native_module, pywrap)
+        self._init_pysqlite2()
+        # no type code in pysqlite2
+        self.BINARY = 'XXX'
+        self.STRING = 'XXX'
+        self.DATETIME = 'XXX'
+        self.NUMBER = 'XXX'
+
+    def _init_pysqlite2(self):
+        """initialize pysqlite2 to use mx.DateTime for date and timestamps"""
+        sqlite = self._native_module
+        if hasattr(sqlite, '_mx_initialized'):
+            return
+
+        from mx.DateTime import DateTimeType, strptime
+        from StringIO import StringIO
+
+        def adapt_mxdatetime(mxd):
+            return mxd.strftime('%F %H:%M:%S')
+        sqlite.register_adapter(DateTimeType, adapt_mxdatetime)
+
+        def convert_mxdate(ustr):
+            return strptime(ustr, '%F %H:%M:%S')
+        sqlite.register_converter('date', convert_mxdate)
+
+        def convert_mxdatetime(ustr):
+            return strptime(ustr, '%F %H:%M:%S')
+        sqlite.register_converter('timestamp', convert_mxdatetime)
+
+        def convert_mxtime(ustr):
+            return strptime(ustr, '%H:%M')
+        sqlite.register_converter('time', convert_mxtime)
+        
+
+        def adapt_bytea(data):
+            return data.getvalue()
+        sqlite.register_adapter(StringIO, adapt_bytea)
+
+        def convert_bytea(data):
+            return StringIO(data)
+        sqlite.register_converter('bytea', convert_bytea)
+
+        def convert_boolean(ustr):
+            if ustr.lower() == 'false':
+                return False
+            return True
+        sqlite.register_converter('boolean', convert_boolean)
+
+        sqlite._mx_initialized = 1
+
+            
+    def connect(self, host='', database='', user='', password='', port=None):
+        """Handles sqlite connexion format"""
+        sqlite = self._native_module
+        
+        class PySqlite2Cursor(sqlite.Cursor):
+            """cursor adapting usual dict format to pysqlite named format
+            in SQL queries
+            """
+            def execute(self, sql, kwargs=None):
+                if kwargs is not None:
+                    sql = re.sub(r'%\(([^\)]+)\)s', r':\1', sql)
+                    self.__class__.__bases__[0].execute(self, sql, kwargs)
+                else:
+                    self.__class__.__bases__[0].execute(self, sql)
+                    
+        class PySqlite2CnxWrapper:
+            def __init__(self, cnx):
+                self._cnx = cnx
+                
+            def cursor(self):
+                return self._cnx.cursor(PySqlite2Cursor)
+            
+            def __getattr__(self, attrname):
+                return getattr(self._cnx, attrname)
+
+        cnx = sqlite.connect(database, detect_types=sqlite.PARSE_DECLTYPES)
+        return self._wrap_if_needed(PySqlite2CnxWrapper(cnx))
+
+    
 class _SqliteAdapter(DBAPIAdapter):
-    """Simple pyPgSQL Adapter to DBAPI
+    """Simple sqlite Adapter to DBAPI
     """
     def __init__(self, native_module, pywrap=False):
         DBAPIAdapter.__init__(self, native_module, pywrap)
         self.DATETIME = native_module.TIMESTAMP
         
-    def connect(self, host = '', database = '', user = '', password = ''):
+    def connect(self, host='', database='', user='', password='', port=''):
         """Handles sqlite connexion format"""
         cnx = self._native_module.connect(database)
         return self._wrap_if_needed(cnx)
 
 
+# Mysql ##############################################################
+
 class _MySqlDBAdapter(DBAPIAdapter):
     """Simple mysql Adapter to DBAPI
     """
-    def connect(self, host = '', database = '', user = '', password = ''):
-        """Handles psycopg connexion format"""
-        kwargs = {'host' : host, 'db' : database,
-                  'user' : user, 'passwd' : password or None}
+    def connect(self, host='', database='', user='', password='', port='',
+                unicode=False):
+        """Handles mysqldb connexion format
+        the unicode named argument asks to use Unicode objects for strings
+        in result sets and query parameters
+        """
+        kwargs = {'host' : host, 'port' : port, 'db' : database,
+                  'user' : user, 'passwd' : password or None,
+                  'use_unicode' : unicode}
         return self._native_module.connect(**kwargs)
 
 
-_ADAPTERS = {
-    'postgres' : { 'pgdb' : _PgdbAdapter,
-                   'psycopg' : _PsycopgAdapter,
-                   'pyPgSQL.PgSQL' : _PgsqlAdapter,
-                   },
-    'mysql' : { 'MySQLdb' : _MySqlDBAdapter, },
-    'sqlite' : { 'sqlite' : _SqliteAdapter, },
-    }
-
-
-# _AdapterDirectory could be more generic by adding a 'protocol' parameter
-# This one would become an adpater for 'DBAPI' protocol
-class _AdapterDirectory(dict):
-    """A simple dict that registers all adapters"""
-    def register_adapter(self, adapter, driver, modname):
-        """Registers 'adapter' in directory as adapting 'mod'"""
-        try:
-            driver_dict = self[driver]
-        except KeyError:
-            self[driver] = {}
-            
-        # XXX Should we have a list of adapters ?
-        driver_dict[modname] = adapter
-    
-    def adapt(self, database, prefered_drivers = None, pywrap = False):
-        """Returns an dbapi-compliant object based for database"""
-        prefered_drivers = prefered_drivers or PREFERED_DRIVERS
-        module, modname = _import_driver_module(database, prefered_drivers)
-        try:
-            return self[database][modname](module, pywrap=pywrap)
-        except KeyError:
-            raise NoAdapterFound(obj=module)        
-
-    def get_adapter(self, database, modname):
-        try:
-            return self[database][modname]
-        except KeyError:
-            raise NoAdapterFound(None, modname)
-
-ADAPTER_DIRECTORY = _AdapterDirectory(_ADAPTERS)
-del _AdapterDirectory
-
-
-## Helpers for DBMS specific Advanced functionalities ################
+## Helpers for DBMS specific Advanced functionalities #########################
 
 class _GenericAdvFuncHelper:
     """Generic helper, trying to provide generic way to implement
@@ -343,6 +389,10 @@ INSERT INTO %s VALUES (0);''' % (seq_name, seq_name)
         return ('UPDATE %s SET last=last+1;' % seq_name,
                 'SELECT last FROM %s;' % seq_name)
 
+    def sql_temporary_table(self, table_name, table_schema):
+        return "CREATE TEMPORARY TABLE %s (%s);" % (table_name,
+                                                                   table_schema)
+    
     def increment_sequence(self, cursor, seq_name):
         for sql in self.sqls_increment_sequence(seq_name):
             cursor.execute(sql)
@@ -365,7 +415,13 @@ class _PGAdvFuncHelper(_GenericAdvFuncHelper):
     
     def sqls_increment_sequence(self, seq_name):
         return ("SELECT nextval('%s');" % seq_name,)
-
+    
+    def sql_temporary_table(self, table_name, table_schema):
+        return "CREATE TEMPORARY TABLE %s (%s) ON COMMIT DROP;" % (table_name,
+                                                                   table_schema)    
+    def sql_current_date(self):
+        return 'TODAY'
+    
 
 class _SqliteAdvFuncHelper(_GenericAdvFuncHelper):
     """Generic helper, trying to provide generic way to implement
@@ -393,14 +449,84 @@ class _SqliteAdvFuncHelper(_GenericAdvFuncHelper):
     def sql_current_timestamp(self):
         return "DATETIME('now')"
     
+
+## Drivers, Adapters and helpers registries ###################################
+
+
+PREFERED_DRIVERS = {
+    "postgres" : [ 'psycopg', 'psycopg2', 'pgdb', 'pyPgSQL.PgSQL', ],
+    "mysql" : [ 'MySQLdb', ], # 'pyMySQL.MySQL, ],
+    "sqlite" : [ 'pysqlite2.dbapi2', 'sqlite', ],
+    }
+
+_ADAPTERS = {
+    'postgres' : { 'pgdb' : _PgdbAdapter,
+                   'psycopg' : _PsycopgAdapter,
+                   'psycopg2' : _Psycopg2Adapter,
+                   'pyPgSQL.PgSQL' : _PgsqlAdapter,
+                   },
+    'mysql' : { 'MySQLdb' : _MySqlDBAdapter, },
+    'sqlite' : { 'pysqlite2.dbapi2' : _PySqlite2Adapter,
+                 'sqlite' : _SqliteAdapter, },
+    }
+
+# _AdapterDirectory could be more generic by adding a 'protocol' parameter
+# This one would become an adapter for 'DBAPI' protocol
+class _AdapterDirectory(dict):
+    """A simple dict that registers all adapters"""
+    def register_adapter(self, adapter, driver, modname):
+        """Registers 'adapter' in directory as adapting 'mod'"""
+        try:
+            driver_dict = self[driver]
+        except KeyError:
+            self[driver] = {}
+            
+        # XXX Should we have a list of adapters ?
+        driver_dict[modname] = adapter
+    
+    def adapt(self, database, prefered_drivers = None, pywrap = False):
+        """Returns an dbapi-compliant object based for database"""
+        prefered_drivers = prefered_drivers or PREFERED_DRIVERS
+        module, modname = _import_driver_module(database, prefered_drivers)
+        try:
+            return self[database][modname](module, pywrap=pywrap)
+        except KeyError:
+            raise NoAdapterFound(obj=module)        
+
+    def get_adapter(self, database, modname):
+        try:
+            return self[database][modname]
+        except KeyError:
+            raise NoAdapterFound(None, modname)
+
+ADAPTER_DIRECTORY = _AdapterDirectory(_ADAPTERS)
+del _AdapterDirectory
     
 ADV_FUNC_HELPER_DIRECTORY = {'postgres': _PGAdvFuncHelper(),
                              'sqlite': _SqliteAdvFuncHelper(),
                              None: _GenericAdvFuncHelper()}
 
 
-## Main functions ####################################################
+## Main functions #############################################################
     
+def set_prefered_driver(database, module, _drivers=PREFERED_DRIVERS):
+    """sets the prefered driver module for database
+    database is the name of the db engine (postgresql, mysql...)
+    module is the name of the module providing the connect function
+    syntax is (params_func, post_process_func_or_None)
+    _drivers is a optionnal dictionnary of drivers
+    """
+    try:
+        modules = _drivers[database]
+    except KeyError:
+        raise UnknownDriver('Unknown database %s' % database)
+    # Remove module from modules list, and re-insert it in first position
+    try:
+        modules.remove(module)
+    except ValueError:
+        raise UnknownDriver('Unknown module %s for %s' % (module, database))
+    modules.insert(0, module)
+
 def get_adv_func_helper(driver):
     """returns an advanced function helper for the given driver"""
     return ADV_FUNC_HELPER_DIRECTORY.get(driver,
@@ -413,14 +539,14 @@ def get_dbapi_compliant_module(driver, prefered_drivers = None, quiet = False,
         mod = ADAPTER_DIRECTORY.adapt(driver, prefered_drivers, pywrap = pywrap)
     except NoAdapterFound, err:
         if not quiet:
-            msg = 'No Adapter found for %s, returning native module' % err.objname
-            print >> sys.stderr, msg
+            msg = 'No Adapter found for %s, returning native module'
+            print >> sys.stderr, msg % err.objname
         mod = err.adapted_obj
     mod.adv_func_helper = get_adv_func_helper(driver)
     return mod
 
-def get_connection(driver='postgres', host='', database='', user='',
-                  password='', quiet=False, drivers=PREFERED_DRIVERS,
+def get_connection(driver='postgres', host='', database='', user='', 
+                  password='', port='', quiet=False, drivers=PREFERED_DRIVERS,
                   pywrap=False):
     """return a db connexion according to given arguments"""
     module, modname = _import_driver_module(driver, drivers, ['connect'])
@@ -433,6 +559,11 @@ def get_connection(driver='postgres', host='', database='', user='',
         adapted_module = DBAPIAdapter(module, pywrap)
     else:
         adapted_module = adapter(module, pywrap)
-    return adapted_module.connect(host, database, user, password)
-
-get_connexion = deprecated_function(get_connection)
+    if host and not port:
+        try:
+            host, port = host.split(':', 1)
+        except ValueError:
+            pass
+    if port:
+        port = int(port)
+    return adapted_module.connect(host, database, user, password, port=port)
