@@ -15,7 +15,7 @@
 # 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 """Some file / file path manipulation utilities.
 
-:version:   $Revision: 1.26 $  
+:version:   $Revision: 1.28 $  
 :author:    Logilab
 :copyright: 2003-2005 LOGILAB S.A. (Paris, FRANCE)
 :contact:   http://www.logilab.fr/ -- mailto:python-projects@logilab.org
@@ -40,14 +40,14 @@ write_open_mode, ensure_fs_mode, export
 
 from __future__ import nested_scopes
 
-__revision__ = "$Id: fileutils.py,v 1.26 2005/04/14 09:37:57 syt Exp $"
+__revision__ = "$Id: fileutils.py,v 1.28 2006-02-07 13:46:01 adim Exp $"
 __docformat__ = "restructuredtext en"
 
 import sys
 import shutil
 import mimetypes
 from os.path import isabs, isdir, split, exists, walk, normpath, join
-from os import sep, linesep, mkdir, remove, listdir, stat, chmod
+from os import sep, mkdir, remove, listdir, stat, chmod
 from stat import ST_MODE, S_IWRITE
 from cStringIO import StringIO
 from warnings import warn
@@ -117,7 +117,7 @@ def get_mode(*args, **kwargs):
          'write_open_mode() instead', DeprecationWarning)
     return write_open_mode(*args, **kwargs)
 
-def ensure_fs_mode(filepath, desired_mode):
+def ensure_fs_mode(filepath, desired_mode=S_IWRITE):
     """check that the given file has the given mode(s) set, else try to
     set it
 
