@@ -1,3 +1,4 @@
+# -*- coding: iso-8859-1 -*-
 # Copyright (c) 2003-2005 LOGILAB S.A. (Paris, FRANCE).
 # http://www.logilab.fr/ -- mailto:contact@logilab.fr
 #
@@ -15,7 +16,7 @@
 # 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 """Python modules manipulation utility functions.
 
-:version:   $Revision: 1.47 $  
+:version:   $Revision: 1.53 $  
 :author:    Logilab
 :copyright: 2003-2005 LOGILAB S.A. (Paris, FRANCE)
 :contact:   http://www.logilab.fr/ -- mailto:python-projects@logilab.org
@@ -34,7 +35,7 @@
 
 from __future__ import nested_scopes
 
-__revision__ = "$Id: modutils.py,v 1.47 2005/04/07 15:42:32 syt Exp $"
+__revision__ = "$Id: modutils.py,v 1.53 2006-03-06 08:05:11 syt Exp $"
 __docformat__ = "restructuredtext en"
 
 import sys
@@ -87,7 +88,7 @@ def load_module_from_name(dotted_name, path=None, use_sys=1):
     return load_module_from_modpath(dotted_name.split('.'), path, use_sys)
 
 
-def load_module_from_modpath(parts, path=None, use_sys=1, _prefix=None):
+def load_module_from_modpath(parts, path=None, use_sys=1):
     """load a python module from it's splitted name
 
     :type parts: list(str) or tuple(str)
@@ -111,22 +112,25 @@ def load_module_from_modpath(parts, path=None, use_sys=1, _prefix=None):
     :rtype: module
     :return: the loaded module
     """
-    if _prefix is None and use_sys:
-        # make tricks like "import os.path" working
-        try:
-            return sys.modules['.'.join(parts)]
-        except KeyError:
-            pass
-    mp_file, mp_filename, mp_desc = find_module(parts[0], path)
-    if _prefix:
-        name = '%s.%s' % (_prefix, parts[0])
-    else:
-        name = parts[0]
-    module = load_module(name, mp_file, mp_filename, mp_desc)
-    if len(parts) == 1:
-        return module
-    return load_module_from_modpath(parts[1:], [dirname(module.__file__)],
-                                  use_sys, name)
+    modpath = []
+    prevmodule = None
+    for part in parts:
+        modpath.append(part)
+        curname = ".".join(modpath)
+        module = None
+        if use_sys:
+            module = sys.modules.get( curname )
+        if module is None:
+            mp_file, mp_filename, mp_desc = find_module(part, path)
+            module = load_module( curname, mp_file, mp_filename, mp_desc)
+        if prevmodule:
+            setattr(prevmodule, part, module)
+        _file = getattr( module, "__file__", "" )
+        if not _file and len(modpath) != len(parts):
+            raise ImportError("no module in %s" % ".".join( parts[len(modpath):] ) )
+        path = [dirname( _file )]
+        prevmodule = module
+    return module
 
 load_module_from_parts = load_module_from_modpath # backward compat
 
@@ -155,14 +159,15 @@ def modpath_from_file(filename):
                 continue
             mod_path = [module for module in base[len(path):].split(os.sep)
                         if module]
-            if len(mod_path) > 1:
-                if not _has_init(join(path, mod_path[0])):
-                    continue
+            for part in mod_path[:-1]:
+                path = join(path, part)
+                if not _has_init(path):
+                    break
+            else:
                 break
-            break
     else:
         raise ImportError('Unable to find module for %s in %s' % (
-            base, ', \n'.join(sys.path)))
+            filename, ', \n'.join(sys.path)))
     return mod_path
 
 
@@ -235,6 +240,9 @@ def get_module_part(dotted_name, context_file=None):
     :return:
       the module part of the name or None if we have not been able at
       all to import the given name
+
+    XXX: deprecated, since it doesn't handle package precedence over module
+    (see #10066)
     """
     # os.path trick
     if dotted_name.startswith('os.path'):
@@ -309,7 +317,7 @@ def get_modules(package, src_directory, blacklist=('CVS', '.svn', 'debian')):
 
 
 
-def get_module_files(src_directory, blacklist=('CVS','.svn', 'debian')):
+def get_module_files(src_directory, blacklist=('CVS', '.svn', 'debian')):
     """given a package directory return a list of all available python
     module's files in the package and its subpackages
 
@@ -442,6 +450,8 @@ def is_relative(modname, from_file):
     """
     if not isdir(from_file):
         from_file = dirname(from_file)
+    if from_file in sys.path:
+        return False
     try:
         find_module(modname.split('.')[0], [from_file])
         return True
