@@ -1,7 +1,7 @@
 """Table management module
 """
 
-__revision__ = '$Id: table.py,v 1.15 2005/08/17 14:48:36 alf Exp $'
+__revision__ = '$Id: table.py,v 1.18 2006-04-09 22:30:53 nico Exp $'
 
 from warnings import warn
 
@@ -25,18 +25,22 @@ class Table(object):
             self.create_rows(row_names)
         
     def _next_row_name(self):
-        return 'row%s' % len(self.row_names)
+        return 'row%s' % (len(self.row_names)+1)
 
     def __iter__(self):
         return iter(self.data)
     
     def __eq__(self, other):
-        if other is not None:
-            return self.data == other.data
-        return False
+        if other is None:
+            return False
+        else:
+            return list(self) == list(other)
 
     def __ne__(self, other):
         return not self == other
+
+    def __len__(self):
+        return len(self.row_names)
     
     ## Rows / Columns creation #################################################
     def create_rows(self, row_names):
@@ -75,7 +79,7 @@ class Table(object):
             col_index = self.col_names.index(col_id)
             self.sort_by_column_index(col_index, method)
         except ValueError:
-            raise KeyError("Col (%s) not found in table"%(col_id))
+            raise KeyError("Col (%s) not found in table" % (col_id))
         
 
     def sort_by_column_index(self, col_index, method = 'asc'):
@@ -109,8 +113,10 @@ class Table(object):
             ptr = groups
             for col_index in col_indexes[:-1]:
                 ptr = ptr.setdefault(row[col_index], {})
-            ptr = ptr.setdefault(row[col_indexes[-1]], [])
-            ptr.append(tuple(row))
+            ptr = ptr.setdefault(row[col_indexes[-1]],
+                                 Table(default_value=self.default_value,
+                                       col_names=self.col_names))
+            ptr.append_row(tuple(row))
         return groups
 
     def select(self, colname, value):
@@ -141,13 +147,13 @@ class Table(object):
         try:
             row_index = self.row_names.index(row_id)
         except ValueError:
-            raise KeyError("Row (%s) not found in table"%(row_id))
+            raise KeyError("Row (%s) not found in table" % (row_id))
         else:
             try:
                 col_index = self.col_names.index(col_id)
                 self.data[row_index][col_index] = data
             except ValueError:
-                raise KeyError("Column (%s) not found in table"%(col_id))
+                raise KeyError("Column (%s) not found in table" % (col_id))
     
     
     def set_row(self, row_index, row_data):
@@ -170,7 +176,7 @@ class Table(object):
             row_index = self.row_names.index(row_id)
             self.set_row(row_index, row_data)
         except ValueError:
-            raise KeyError('Row (%s) not found in table'%(row_id))
+            raise KeyError('Row (%s) not found in table' % (row_id))
         
 
     def append_row(self, row_data, row_name=None):
@@ -213,7 +219,7 @@ class Table(object):
             row_index = self.row_names.index(row_id)
             self.delete_row(row_index)
         except ValueError:
-            raise KeyError('Row (%s) not found in table'%(row_id))
+            raise KeyError('Row (%s) not found in table' % (row_id))
     
 
     def set_column(self, col_index, col_data):
@@ -238,7 +244,7 @@ class Table(object):
             col_index = self.col_names.index(col_id)
             self.set_column(col_index, col_data)
         except ValueError:
-            raise KeyError('Column (%s) not found in table'%(col_id))
+            raise KeyError('Column (%s) not found in table' % (col_id))
         
 
     def append_column(self, col_data, col_name):
@@ -281,20 +287,76 @@ class Table(object):
             col_index = self.col_names.index(col_id)
             self.delete_column(col_index)
         except ValueError:
-            raise KeyError('Column (%s) not found in table'%(col_id))
+            raise KeyError('Column (%s) not found in table' % (col_id))
 
     
     ## The 'getter' part #######################################################
 
-    def __getitem__(self, row_index):
+    def get_shape(self):
+        """Returns a tuple which represents the table's shape
+        """    
+        return len(self.row_names), len(self.col_names)
+    shape = property(get_shape)
+    
+    def __getitem__(self, indices):
         """provided for convenience"""
-        return self.data[row_index]
+        rows, multirows = None, False
+        cols, multicols = None, False
+        if isinstance(indices, tuple):
+            rows = indices[0]
+            if len(indices) > 1:
+                cols = indices[1]
+        else:
+            rows = indices
+        # define row slice
+        if isinstance(rows,str):
+            try:
+                rows = self.row_names.index(rows)
+            except ValueError:
+                raise KeyError("Row (%s) not found in table" % (rows))
+        if isinstance(rows,int):
+            rows = slice(rows,rows+1)
+            multirows = False
+        else:
+            rows = slice(None)
+            multirows = True
+        # define col slice
+        if isinstance(cols,str):
+            try:
+                cols = self.col_names.index(cols)
+            except ValueError:
+                raise KeyError("Column (%s) not found in table" % (cols))
+        if isinstance(cols,int):
+            cols = slice(cols,cols+1)
+            multicols = False
+        else:
+            cols = slice(None)
+            multicols = True
+        # get sub-table
+        tab = Table()
+        tab.default_value = self.default_value
+        tab.create_rows(self.row_names[rows])
+        tab.create_columns(self.col_names[cols])
+        for idx,row in enumerate(self.data[rows]):
+            tab.set_row(idx, row[cols])
+        if multirows :
+            if multicols:
+                return tab
+            else:
+                return [item[0] for item in tab.data]
+        else:
+            if multicols:
+                return tab.data[0]
+            else:
+                return tab.data[0][0]
 
     def get_dimensions(self):
-        """Returns a tuple which represents the table's size
+        """Returns a tuple which represents the table's shape
         """
-        return len(self.row_names), len(self.col_names)
-        
+        warn('table.get_dimensions() is deprecated, use table.shape instead',
+             DeprecationWarning)
+        return self.shape
+
     def get_element(self, row_index, col_index):
         """Returns the element at [row_index][col_index]
         """
@@ -303,39 +365,49 @@ class Table(object):
         return self.data[row_index][col_index]
 
     def get_cell(self, row_index, col_index):
+        warn('table.get_cell(i,j) is deprecated, use table[i,j] instead',
+             DeprecationWarning)
         return self.data[row_index][col_index]
         
     def get_cell_by_ids(self, row_id, col_id):
         """Returns the element at [row_id][col_id]
         """
+        warn('table.get_cell_by_ids(i,j) is deprecated, use table[i,j] instead',
+             DeprecationWarning)
         try:
             row_index = self.row_names.index(row_id)
         except ValueError:
-            raise KeyError("Row (%s) not found in table"%(row_id))
+            raise KeyError("Row (%s) not found in table" % (row_id))
         else:
             try:
                 col_index = self.col_names.index(col_id)
             except ValueError:
-                raise KeyError("Column (%s) not found in table"%(col_id))
+                raise KeyError("Column (%s) not found in table" % (col_id))
         return self.data[row_index][col_index]
 
     def get_row(self, row_index):
         """Returns the 'row_index' row
         """
+        warn('table.get_row(i) is deprecated, use table[i] instead',
+             DeprecationWarning)
         return self.data[row_index]
 
     def get_row_by_id(self, row_id):
         """Returns the 'row_id' row
         """
+        warn('table.get_row_by_id(i) is deprecated, use table[i] instead',
+             DeprecationWarning)
         try:
             row_index = self.row_names.index(row_id)
         except ValueError:
-            raise KeyError("Row (%s) not found in table"%(row_id))
+            raise KeyError("Row (%s) not found in table" % (row_id))
         return self.get_row(row_index)
 
     def get_column(self, col_index, distinct=False):
         """Returns the 'col_index' col
         """
+        warn('table.get_column(i) is deprecated, use table[:,i] instead',
+             DeprecationWarning)
         col = [row[col_index] for row in self.data]
         if distinct:
             return set(col)
@@ -345,16 +417,20 @@ class Table(object):
     def get_column_by_id(self, col_id, distinct=False):
         """Returns the 'col_id' col
         """
+        warn('table.get_column_by_id(i) is deprecated, use table[:,i] instead',
+             DeprecationWarning)
         try:
             col_index = self.col_names.index(col_id)
         except ValueError:
-            raise KeyError("Column (%s) not found in table"%(col_id))
+            raise KeyError("Column (%s) not found in table" % (col_id))
         return self.get_column(col_index, distinct)
     
 
     def get_rows(self):
         """Returns all the rows in the table
         """
+        warn('table.get_rows() is deprecated, just iterate over table instead',
+             DeprecationWarning)
         return self.data
 
 
@@ -372,7 +448,7 @@ class Table(object):
         
 
     def transpose(self):
-        """Keeps the self object intact, and returns the transpoed (rotated)
+        """Keeps the self object intact, and returns the transposed (rotated)
         table.
         """
         transposed = Table()
@@ -380,7 +456,6 @@ class Table(object):
         transposed.create_columns(self.row_names)
         for col_index, column in enumerate(self.get_columns()):
             transposed.set_row(col_index, column)
-
         return transposed
 
 
@@ -426,7 +501,10 @@ class Table(object):
         return '\n'.join(lines)
     
 
-    def __str__(self):
+    def __repr__(self):
+        return repr(self.data)
+
+    def as_text(self):
         data = []
         # We must convert cells into strings before joining them
         for row in self.data:
@@ -619,7 +697,7 @@ class TableStyleSheet:
         """
         cell_list = ['%d_%d'%(row_index, index) for index in range(start_col,
                                                                    end_col + 1)]
-        rule = '%d_%d='%dest_cell + '+'.join(cell_list)
+        rule = '%d_%d=' % dest_cell + '+'.join(cell_list)
         self.add_rule(rule)
             
 
@@ -635,7 +713,7 @@ class TableStyleSheet:
         cell_list = ['%d_%d'%(row_index, index) for index in range(start_col,
                                                                    end_col + 1)]
         num = (end_col - start_col + 1)
-        rule = '%d_%d='%dest_cell + '('+'+'.join(cell_list)+')/%f'%num
+        rule = '%d_%d=' % dest_cell + '('+'+'.join(cell_list)+')/%f'%num
         self.add_rule(rule)
         
 
@@ -650,7 +728,7 @@ class TableStyleSheet:
         """        
         cell_list = ['%d_%d'%(index, col_index) for index in range(start_row,
                                                                    end_row + 1)]
-        rule = '%d_%d='%dest_cell + '+'.join(cell_list)
+        rule = '%d_%d=' % dest_cell + '+'.join(cell_list)
         self.add_rule(rule)
         
     
@@ -666,7 +744,7 @@ class TableStyleSheet:
         cell_list = ['%d_%d'%(index, col_index) for index in range(start_row,
                                                                    end_row + 1)]
         num = (end_row - start_row + 1)
-        rule = '%d_%d='%dest_cell + '('+'+'.join(cell_list)+')/%f'%num
+        rule = '%d_%d=' % dest_cell + '('+'+'.join(cell_list)+')/%f'%num
         self.add_rule(rule)
 
 
@@ -718,7 +796,7 @@ class TableCellRenderer:
         Rendering properties will be searched using the
         *table_style.get_xxx_by_index(col_index)' methods
 
-        **This method should be overriden in the derived renderer classes.**
+        **This method should be overridden in the derived renderer classes.**
         """
         return content
 
@@ -758,10 +836,6 @@ class DocbookRenderer(TableCellRenderer):
     """Defines how to render a cell for a docboook table
     """
 
-    def __init__(self, **properties):
-        TableCellRenderer.__init__(self, **properties)
-
-
     def define_col_header(self, col_index, table_style):
         """Computes the colspec element according to the style
         """
@@ -783,7 +857,7 @@ class DocbookRenderer(TableCellRenderer):
                        (alignment, cell_content)
         except KeyError:
             # KeyError <=> Default alignment
-            return "<entry>%s</entry>\n"%cell_content
+            return "<entry>%s</entry>\n" % cell_content
 
 
 class TableWriter:
