@@ -2,7 +2,7 @@
 
 Some file / path manipulation utilities
 """
-__revision__ = "$Id: unittest_fileutils.py,v 1.21 2005/08/05 11:50:43 adim Exp $"
+__revision__ = "$Id: unittest_fileutils.py,v 1.22 2006-01-03 15:31:16 syt Exp $"
 
 import unittest
 import sys, os, tempfile, shutil
@@ -79,7 +79,7 @@ class GetByExtTC(TestCase):
                                                        ]])
         
     def test_exclude_base_dir(self):
-        self.assertEquals(files_by_ext(DATA_DIR, include_exts=('.py',), exclude_dirs=(DATA_DIR)),
+        self.assertEquals(files_by_ext(DATA_DIR, include_exts=('.py',), exclude_dirs=(DATA_DIR,)),
                           [])
 
 class ExportTC(TestCase):
