@@ -14,18 +14,22 @@
 unit tests for module modutils (module manipulation utilities)
 """
 
-__revision__ = "$Id: unittest_modutils.py,v 1.24 2005/02/07 18:04:23 syt Exp $"
+__revision__ = "$Id: unittest_modutils.py,v 1.26 2006-02-16 10:58:40 ludal Exp $"
 
-import unittest
 import sys
+try:
+    __file__
+except NameError:
+    __file__ = sys.argv[0]
 
+from logilab.common.testlib import TestCase, unittest_main
 from logilab.common import modutils
 
 from os import path
 from logilab import common
 from logilab.common import tree
     
-class load_module_from_name_tc(unittest.TestCase):
+class load_module_from_name_tc(TestCase):
     """ load a python module from it's name """
     
     def test_knownValues_load_module_from_name_1(self):
@@ -39,7 +43,7 @@ class load_module_from_name_tc(unittest.TestCase):
                           modutils.load_module_from_name, 'os.path', use_sys=0)    
 
 
-class get_module_part_tc(unittest.TestCase):
+class get_module_part_tc(TestCase):
     """given a dotted name return the module part of the name"""
     
     def test_knownValues_get_module_part_1(self):
@@ -63,7 +67,7 @@ class get_module_part_tc(unittest.TestCase):
         self.assertEqual(modutils.get_module_part('sys.path', '__file__'), 'sys')
 
     
-class modpath_from_file_tc(unittest.TestCase):
+class modpath_from_file_tc(TestCase):
     """ given an absolute file path return the python module's path as a list """
     
     def test_knownValues_modpath_from_file_1(self):
@@ -74,7 +78,7 @@ class modpath_from_file_tc(unittest.TestCase):
         self.assertRaises(Exception, modutils.modpath_from_file, '/turlututu')
 
 
-class file_from_modpath_tc(unittest.TestCase):
+class file_from_modpath_tc(TestCase):
     """given a mod path (i.e. splited module / package name), return the
     corresponding file, giving priority to source file over precompiled file
     if it exists"""
@@ -105,7 +109,7 @@ class file_from_modpath_tc(unittest.TestCase):
     def test_raise_file_from_modpath_Exception(self):
         self.assertRaises(ImportError, modutils.file_from_modpath, ['turlututu'])
 
-class get_source_file_tc(unittest.TestCase):
+class get_source_file_tc(TestCase):
     
     def test(self):
         from os import path
@@ -115,7 +119,7 @@ class get_source_file_tc(unittest.TestCase):
     def test_raise(self):
         self.assertRaises(modutils.NoSourceFile, modutils.get_source_file,'whatever')
     
-class is_standard_module_tc(unittest.TestCase):
+class is_standard_module_tc(TestCase):
     """
     return true if the module may be considered as a module from the standard
     library
@@ -141,7 +145,7 @@ class is_standard_module_tc(unittest.TestCase):
         self.assertEqual(modutils.is_standard_module('data.module', (path.abspath('data'),)), True)
 
     
-class is_relative_tc(unittest.TestCase):
+class is_relative_tc(TestCase):
     
     def test_knownValues_is_relative_1(self):
         self.assertEqual(modutils.is_relative('modutils', common.__path__[0]), True)
@@ -153,7 +157,7 @@ class is_relative_tc(unittest.TestCase):
         self.assertEqual(modutils.is_relative('logilab.common.modutils',
                                               common.__path__[0]), False)
     
-class get_modules_tc(unittest.TestCase):
+class get_modules_tc(TestCase):
     
     def test_knownValues_get_modules_1(self): #  XXXFIXME: TOWRITE
         """given a directory return a list of all available python modules, even
@@ -167,7 +171,7 @@ class get_modules_tc(unittest.TestCase):
                           'data.nonregr'])
 
 
-class get_modules_files_tc(unittest.TestCase):
+class get_modules_files_tc(TestCase):
     
     def test_knownValues_get_module_files_1(self): #  XXXFIXME: TOWRITE
         """given a directory return a list of all available python module's files, even
@@ -179,6 +183,15 @@ class get_modules_files_tc(unittest.TestCase):
         self.assertEqual(modules,
                          [path.join('data', x) for x in ['__init__.py', 'module.py', 'module2.py', 'noendingnewline.py', 'nonregr.py']])
 
+    def test_load_module_set_attribute(self):
+        import logilab.common
+        import logilab
+        del logilab.common
+        del sys.modules['logilab.common']
+        m = modutils.load_module_from_modpath(['logilab','common'])
+        self.assert_( hasattr(logilab,'common') )
+        self.assert_( m is logilab.common )
+
 from logilab.common.testlib import DocTest
 class ModuleDocTest(DocTest):
     """test doc test in this module"""
@@ -187,4 +200,4 @@ del DocTest # necessary if we don't want it to be executed (we don't...)
 
 
 if __name__ == '__main__':
-    unittest.main()
+    unittest_main()
