@@ -2,114 +2,130 @@
 Unittests for table management
 """
 
-__revision__ = '$Id: unittest_table.py,v 1.11 2005/08/19 16:16:23 nico Exp $'
+__revision__ = '$Id: unittest_table.py,v 1.13 2006-04-09 22:30:53 nico Exp $'
 
-import unittest
 import sys
+from cStringIO import StringIO
+
+from logilab.common.testib import TestCase, unittest_main
 from logilab.common.table import Table, TableStyleSheet, DocbookTableWriter, \
      DocbookRenderer, TableStyle, TableWriter, TableCellRenderer
 from logilab.common.compat import set
-from cStringIO import StringIO
 
-class TableTC(unittest.TestCase):
-    """Table TestCase class
-    """
+class TableTC(TestCase):
+    """Table TestCase class"""
 
     def setUp(self):
-        """Creates a default table
-        """
+        """Creates a default table"""
         self.table = Table()
         self.table.create_rows(['row1', 'row2', 'row3'])
         self.table.create_columns(['col1', 'col2'])
 
+    def test_valeur_scalaire(self):
+        tab = Table()
+        tab.create_columns(['col1'])
+        tab.append_row([1])
+        self.assertEquals(tab, [[1]])
+        tab.append_row([2])
+        self.assertEquals(tab[0,0], 1)
+        self.assertEquals(tab[1,0], 2)
 
+    def test_valeur_ligne(self):
+        tab = Table()
+        tab.create_columns(['col1','col2'])
+        tab.append_row([1,2])
+        self.assertEquals(tab, [[1,2]])
+
+    def test_valeur_colonne(self):
+        tab = Table()
+        tab.create_columns(['col1'])
+        tab.append_row([1])
+        tab.append_row([2])
+        self.assertEquals(tab, [[1],[2]])
+        self.assertEquals(tab[:,0], [1,2])
+        
     def test_indexation(self):
         """we should be able to use [] to access rows"""
-        self.assert_(self.table[0] is self.table.data[0])
-        self.assert_(self.table[1] is self.table.data[1])
+        self.assert_(self.table[0] == self.table.data[0])
+        self.assert_(self.table[1] == self.table.data[1])
 
     def test_iterable(self):
         """test iter(table)"""
         it = iter(self.table)
-        self.assert_(it.next() is self.table.data[0])
-        self.assert_(it.next() is self.table.data[1])
+        self.assert_(it.next() == self.table.data[0])
+        self.assert_(it.next() == self.table.data[1])
 
     def test_get_rows(self):
         """tests Table.get_rows()"""
-        self.assertEquals(self.table.get_rows(), [[0, 0], [0, 0], [0, 0]])
+        self.assertEquals(self.table, [[0, 0], [0, 0], [0, 0]])
+        self.assertEquals(self.table[:], [[0, 0], [0, 0], [0, 0]])
         self.table.insert_column(1, range(3), 'supp')
-        self.assertEquals(self.table.get_rows(), [[0, 0, 0], [0, 1, 0], [0, 2, 0]])
+        self.assertEquals(self.table, [[0, 0, 0], [0, 1, 0], [0, 2, 0]])
+        self.assertEquals(self.table[:], [[0, 0, 0], [0, 1, 0], [0, 2, 0]])
         
     def test_get_cells(self):
         self.table.insert_column(1, range(3), 'supp')
-        self.assertEquals(self.table.get_cell(0, 1), 0)
-        self.assertEquals(self.table.get_cell(1, 1), 1)
-        self.assertEquals(self.table.get_cell(2, 1), 2)
-        self.assertEquals(self.table.get_cell_by_ids('row1', 'supp'), 0)
-        self.assertEquals(self.table.get_cell_by_ids('row2', 'supp'), 1)
-        self.assertEquals(self.table.get_cell_by_ids('row3', 'supp'), 2)
-        self.assertRaises(KeyError, self.table.get_cell_by_ids, 'row1', 'foo')
-        self.assertRaises(KeyError, self.table.get_cell_by_ids, 'foo', 'bar')
+        self.assertEquals(self.table[0,1], 0)
+        self.assertEquals(self.table[1,1], 1)
+        self.assertEquals(self.table[2,1], 2)
+        self.assertEquals(self.table['row1', 'supp'], 0)
+        self.assertEquals(self.table['row2', 'supp'], 1)
+        self.assertEquals(self.table['row3', 'supp'], 2)
+        self.assertRaises(KeyError, self.table.__getitem__, ('row1', 'foo'))
+        self.assertRaises(KeyError, self.table.__getitem__, ('foo', 'bar'))
         
-    def test_dimensions(self):
-        """tests table dimensions"""
-        self.assertEquals(self.table.get_dimensions(), (3, 2))
+    def test_shape(self):
+        """tests table shape"""
+        self.assertEquals(self.table.shape, (3, 2))
         self.table.insert_column(1, range(3), 'supp')
-        self.assertEquals(self.table.get_dimensions(), (3, 3))
+        self.assertEquals(self.table.shape, (3, 3))
         
     def test_set_column(self):
         """Tests that table.set_column() works fine.
         """
         self.table.set_column(0, range(3))
-        self.assertEquals(self.table.get_cell(0,0), 0)
-        self.assertEquals(self.table.get_cell(1,0), 1)
-        self.assertEquals(self.table.get_cell(2,0), 2)
+        self.assertEquals(self.table[0,0], 0)
+        self.assertEquals(self.table[1,0], 1)
+        self.assertEquals(self.table[2,0], 2)
 
     def test_set_column_by_id(self):
         """Tests that table.set_column_by_id() works fine.
         """
         self.table.set_column_by_id('col1', range(3))
-        self.assertEquals(self.table.get_cell(0,0), 0)
-        self.assertEquals(self.table.get_cell(1,0), 1)
-        self.assertEquals(self.table.get_cell(2,0), 2)
+        self.assertEquals(self.table[0,0], 0)
+        self.assertEquals(self.table[1,0], 1)
+        self.assertEquals(self.table[2,0], 2)
         self.assertRaises(KeyError, self.table.set_column_by_id, 'col123', range(3))
 
     def test_cells_ids(self):
         """tests that we can access cells by giving row/col ids"""
         self.assertRaises(KeyError, self.table.set_cell_by_ids, 'row12', 'col1', 12)
         self.assertRaises(KeyError, self.table.set_cell_by_ids, 'row1', 'col12', 12)
-        self.assertEquals(self.table.get_cell(0,0), 0)
+        self.assertEquals(self.table[0,0], 0)
         self.table.set_cell_by_ids('row1', 'col1', 'DATA')
-        self.assertEquals(self.table.get_cell(0,0), 'DATA')
+        self.assertEquals(self.table[0,0], 'DATA')
         self.assertRaises(KeyError, self.table.set_row_by_id, 'row12', [])
         self.table.set_row_by_id('row1', ['1.0', '1.1'])
-        self.assertEquals(self.table.get_cell(0,0), '1.0')
+        self.assertEquals(self.table[0,0], '1.0')
 
     def test_insert_row(self):
         """tests a row insertion"""
         tmp_data = ['tmp1', 'tmp2']
         self.table.insert_row(1, tmp_data, 'tmprow')
-        self.assertEquals(self.table.get_row(1), tmp_data)
-        self.assertEquals(self.table.get_row_by_id('tmprow'), tmp_data)
+        self.assertEquals(self.table[1], tmp_data)
+        self.assertEquals(self.table['tmprow'], tmp_data)
         self.table.delete_row_by_id('tmprow')
         self.assertRaises(KeyError, self.table.delete_row_by_id, 'tmprow')
-        self.assertEquals(self.table.get_row(1), [0, 0])
-        self.assertRaises(KeyError, self.table.get_row_by_id, 'tmprow')
+        self.assertEquals(self.table[1], [0, 0])
+        self.assertRaises(KeyError, self.table.__getitem__, 'tmprow')
         
     def test_get_column(self):
         """Tests that table.get_column() works fine.
         """
         self.table.set_cell(0, 1, 12)
         self.table.set_cell(2, 1, 13)
-        self.assertEquals(self.table.get_column(1), [12,0,13])
-        self.assertEquals(self.table.get_column_by_id('col2'), [12,0,13])
-
-    def test_get_column_distinct(self):
-        """tests get_column(distinct=True)"""
-        self.table.set_cell(0, 1, 12)
-        self.table.set_cell(2, 1, 12)
-        self.assertEquals(self.table.get_column(1, distinct=True), set([12,0]))
-        self.assertEquals(self.table.get_column_by_id('col2', distinct=True), set([12,0]))
+        self.assertEquals(self.table[:,1], [12,0,13])
+        self.assertEquals(self.table[:,'col2'], [12,0,13])
 
     def test_get_columns(self):
         """Tests if table.get_columns() works fine.
@@ -122,7 +138,7 @@ class TableTC(unittest.TestCase):
         """Tests that table.insert_column() works fine.
         """
         self.table.insert_column(1, range(3), "inserted_column")
-        self.assertEquals(self.table.get_column(1), [0,1,2])
+        self.assertEquals(self.table[:,1], [0,1,2])
         self.assertEquals(self.table.col_names,
                           ['col1', 'inserted_column', 'col2'])
 
@@ -131,7 +147,7 @@ class TableTC(unittest.TestCase):
         """
         self.table.delete_column(1)
         self.assertEquals(self.table.col_names, ['col1'])
-        self.assertEquals(self.table.get_column(0), [0,0,0])
+        self.assertEquals(self.table[:,0], [0,0,0])
         self.assertRaises(KeyError, self.table.delete_column_by_id, 'col2')
         self.table.delete_column_by_id('col1')
         self.assertEquals(self.table.col_names, [])
@@ -175,7 +191,7 @@ class TableTC(unittest.TestCase):
         str(self.table)
 
 
-class GroupByTC(unittest.TestCase):
+class GroupByTC(TestCase):
     """specific test suite for groupby()"""
     def setUp(self):
         t = Table()
@@ -188,7 +204,6 @@ class GroupByTC(unittest.TestCase):
         t.append_row(['date3', 'ing1', 'task3', 0.3])        
         self.table = t
 
-    
     def test_single_groupby(self):
         """tests groupby() on several columns"""
         grouped = self.table.groupby('date')
@@ -230,7 +245,7 @@ class GroupByTC(unittest.TestCase):
             ('date1', 'ing1', 'task3', 0.3),
             ])
 
-class TableStyleSheetTC(unittest.TestCase):
+class TableStyleSheetTC(TestCase):
     """The Stylesheet test case
     """
     def setUp(self):
@@ -254,8 +269,7 @@ class TableStyleSheetTC(unittest.TestCase):
         self.stylesheet.add_rule(rule)
         self.table.set_row(0, [3,4,0])
         self.table.apply_stylesheet(self.stylesheet)
-        self.assertEquals(self.table.get_row(0),
-                          [3,4,5])
+        self.assertEquals(self.table[0], [3,4,5])
         self.assertEquals(len(self.stylesheet.rules), 1)
         self.stylesheet.add_rule('some bad rule with bad syntax')
         self.assertEquals(len(self.stylesheet.rules), 1, "Ill-formed rule mustn't be added")
@@ -274,7 +288,7 @@ class TableStyleSheetTC(unittest.TestCase):
         self.table.set_row(0, [10,20,0])
         self.stylesheet.add_rowavg_rule((0,2), 0, 0, 1)
         self.table.apply_stylesheet(self.stylesheet)
-        val = self.table.get_cell(0,2)
+        val = self.table[0,2]
         self.assert_(int(val) == 15)
         
 
@@ -284,7 +298,7 @@ class TableStyleSheetTC(unittest.TestCase):
         self.table.set_row(0, [10,20,0])
         self.stylesheet.add_rowsum_rule((0,2), 0, 0, 1)
         self.table.apply_stylesheet(self.stylesheet)
-        val = self.table.get_cell(0,2)
+        val = self.table[0,2]
         self.assert_(val == 30)
         
 
@@ -296,7 +310,7 @@ class TableStyleSheetTC(unittest.TestCase):
         self.table.create_row('row3')
         self.stylesheet.add_colavg_rule((2,0), 0, 0, 1)
         self.table.apply_stylesheet(self.stylesheet)
-        val = self.table.get_cell(2,0)
+        val = self.table[2,0]
         self.assert_(int(val) == 11)
         
 
@@ -308,12 +322,12 @@ class TableStyleSheetTC(unittest.TestCase):
         self.table.create_row('row3')
         self.stylesheet.add_colsum_rule((2,0), 0, 0, 1)
         self.table.apply_stylesheet(self.stylesheet)
-        val = self.table.get_cell(2,0)
+        val = self.table[2,0]
         self.assert_(val == 22)
 
 
 
-class TableStyleTC(unittest.TestCase):
+class TableStyleTC(TestCase):
     """Test suite for TableSuite"""
     def setUp(self):
         self.table = Table()
@@ -351,7 +365,7 @@ class TableStyleTC(unittest.TestCase):
             self.assertEquals(igetter(1), 'FOO')
         
 
-class RendererTC(unittest.TestCase):
+class RendererTC(TestCase):
     """Test suite for DocbookRenderer"""
     def setUp(self):
         self.renderer = DocbookRenderer(alignment = True)
@@ -390,8 +404,7 @@ class RendererTC(unittest.TestCase):
         self.assertEquals(cell_content, '12 EUR')
 
 
-from logilab.common import testlib
-class DocbookTableWriterTC(testlib.TestCase):
+class DocbookTableWriterTC(TestCase):
     """TestCase for table's writer"""
     def setUp(self):
         self.stream = StringIO()
@@ -412,4 +425,4 @@ class DocbookTableWriterTC(testlib.TestCase):
     
     
 if __name__ == '__main__':
-    unittest.main()
+    unittest_main()
