@@ -17,7 +17,7 @@
  base class to represent tree structure
 """
 
-__revision__ = "$Id: tree.py,v 1.20 2005/08/08 10:42:24 adim Exp $"
+__revision__ = "$Id: tree.py,v 1.24 2006-02-24 10:40:21 nico Exp $"
 
 import sys
 from warnings import warn
@@ -199,8 +199,16 @@ class Node :
             c.flatten(_list)
         return _list
 
-        
-class VNode(VisitedMixIn, Node):
+    def lineage(self):
+        """
+        return list of parents up to root node
+        """
+        lst = [self]
+        if self.parent is not None:
+            lst.extend(self.parent.lineage())
+        return lst
+    
+class VNode(Node, VisitedMixIn):
     """a visitable node
     """
     pass
